package datahub;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.FileCopyUtils;
import org.springframework.util.FileSystemUtils;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.api.client.util.Data;

import datahub.model.*;
import datahub.tools.DataGatherer;
import datahub.firebase.FirebaseDataHandler;
import test.TestData;



@CrossOrigin(origins = { "http://localhost:3000", "http://localhost:4200", "http://localhost:8097" })
@RestController
@RequestMapping(path="/") 
public class DatahubResource {
	
	@Autowired 
	private DatasetFileRepository datasetFileRepository;
	@Autowired
	private CategoryRepository categoryRepository;
	@Autowired
	private DatasetRepository datasetRepository;
	@Autowired
	private UserRepository userRepository;
	
	private static String UPLOADED_FOLDER = "files/users/";
	
	@PostMapping("/uploadMulti") // //new annotation since 4.3
	  public ArrayList<String> multiFileUpload(@RequestParam("files") MultipartFile [] files,
			  @RequestParam("name") String datasetName, 
			  @RequestParam("user") String userEmail,
	          RedirectAttributes redirectAttributes) {

		  ArrayList<String> filesUploaded = new ArrayList<String>();
		  
		  System.out.println(" DATASET NAME ----> "+datasetName);
		  
		  
		  if(!datasetName.isEmpty() && !userEmail.isEmpty()) {
		  
			  String folderPath = UPLOADED_FOLDER + userEmail + "/"+ datasetName;
			  
			  File directory = new File(folderPath);
			  
			  if(!directory.exists())
				  directory.mkdirs();
			  
		      if (files.length == 0) {
		          redirectAttributes.addFlashAttribute("message", "Please select a file to upload");
		          System.out.println("ERROR");
		          return null;
		      }
		
		      try {
		    	  
		    	  System.out.println(files.length);
		    	  
		    	  for(MultipartFile file : files) {
			          // Get the file and save it somewhere
		    		  
		    		  System.out.println(file.getOriginalFilename());
		    		  
			          byte[] bytes = file.getBytes();
			          Path path = Paths.get(folderPath + "/" + file.getOriginalFilename());
			          Files.write(path, bytes);
			          filesUploaded.add(file.getOriginalFilename());
			          
		          
		    	  }
		    	  
		    	  redirectAttributes.addFlashAttribute("message",
		                  "You successfully uploaded the files'");
		
		      } catch (IOException e) {
		          e.printStackTrace();
		      }
		  }

	      //return "redirect:/uploadStatus";
	      return filesUploaded;
	  }
	  
	@PostMapping("/getSampleData") // //new annotation since 4.3
	  public ArrayList<ArrayList<String>> getSampleData(@RequestParam("user") String userEmail,
			  @RequestParam("dataset") String datasetName, 
			  @RequestParam("distribution") String distribution,
	          RedirectAttributes redirectAttributes) {

		ArrayList<ArrayList<String>> sampleData = new ArrayList<ArrayList<String>>();
		  
		  System.out.println(" DATASET NAME ----> "+datasetName);
		  
		  
		  if(!datasetName.isEmpty() && !userEmail.isEmpty()) {
		  
			  String filePath = UPLOADED_FOLDER + userEmail + "/"+ datasetName+"/"+distribution;
			  
			  File file = new File(filePath);
			  
			  if(file.exists())
			  {
				  sampleData = DataGatherer.getSampleData(file, 50);
			  }
		  }
	  
	      return sampleData;
	  }
	
	@PostMapping("/getSampleDataJSON") // //new annotation since 4.3
	  public String getSampleDataJSON(@RequestParam("user") String userEmail,
			  @RequestParam("dataset") String datasetName, 
			  @RequestParam("distribution") String distribution,
	          RedirectAttributes redirectAttributes) {

		String sampleData = "";

		  	  
		  if(!datasetName.isEmpty() && !userEmail.isEmpty()) {
		  
			  String filePath = UPLOADED_FOLDER + userEmail + "/"+ datasetName+"/"+distribution;
			  
			  File file = new File(filePath);
			  
			  if(file.exists())
			  {
				  sampleData = DataGatherer.getJSONRows(file, 25);
			  }
		  }
	  
	      return sampleData;
	  }
	  
	  
	  @GetMapping("/uploadStatus")
	  public String uploadStatus() {
	      return "uploadStatus";
	  }
	  
	  
	  @PostMapping("/getData") // //new annotation since 4.3
	  public Dataset getDatasetData(@RequestParam("folder") String folder,
			  @RequestParam("user") String userEmail,
	          RedirectAttributes redirectAttributes) {

		  Dataset datasetF = new Dataset();
		  
		  //obtener datos del usuario por el email de la bd y asignar al dataset creado
		  
		  
	      if (folder.isEmpty() || userEmail.isEmpty()) {
	          redirectAttributes.addFlashAttribute("message", "Please select a file to upload");
	          return datasetF;
	      }

	      try {
	    	  System.out.println(userEmail + "    ---   "+ folder);
	    	  ArrayList<Dataset> datasets = datasetRepository.getDatasetUserEmailAndName(folder, userEmail);
	    	  
	    	  if(datasets.size() > 0) {
	    		  datasetF = datasets.get(0);
	    	  }
	    	  else {
	    		  
	    		  datasetF.setName(folder);
		    	  datasetF.setOwnerEmail(userEmail);
		    	  datasetF.addDistributions(DataGatherer.getDatasetFilesData(UPLOADED_FOLDER + userEmail + "/" + folder));
	    	  }
	    	  
	    	  ArrayList<User> aux = userRepository.getUserByEmail(userEmail);
    		  
    		  if(aux.size() > 0) {
    			  datasetF.setUser(aux.get(0));
    			  System.out.println("Añado User al dataset --- ID --- "+ aux.get(0).getEmail());
    		  }
	    	  //datasetF=FirebaseDataHandler.getDataset(folder, userEmail);
	    	  datasetF.clearTerms();
	    	  System.out.println("Dataset ID ----> "+datasetF.getId());

	      } catch (Exception e) {
	          e.printStackTrace();
	      }

	      return datasetF;
	  }
	  
	  @PostMapping("/getUserData") // //new annotation since 4.3
	  public User getUserData(@RequestParam("user") String userEmail,
	          RedirectAttributes redirectAttributes) {

		  User u = new User();
		  if(userEmail != null && !userEmail.isEmpty()) {
			  ArrayList<User> aux = userRepository.getUserByEmail(userEmail);
			  
			  
			  if(aux.size() > 0) {
				  u = aux.get(0);
			  }
		  }
		  

	      return u;
	  }
	  
	  @PostMapping("/deleteUser") // //new annotation since 4.3
	  public void deleteUser(@RequestParam("userID") int userID,
			  @RequestParam("userEmail") String userEmail,
			  RedirectAttributes redirectAttributes) {
		  
		  if(userEmail != null && userID > 0) {
			 userRepository.deleteById(userID);
			 FirebaseDataHandler.deleteUser(userEmail);
			 File file = new File(UPLOADED_FOLDER + userEmail);
			 if(file.exists()) {
				 FileSystemUtils.deleteRecursively(file);
			 }
		  }
	  }
	  
	  @RequestMapping(value = "/saveUserData", method = RequestMethod.POST)
	  public String saveUserData(@RequestBody User user,
	                                 RedirectAttributes redirectAttributes) {
		  
	      if (user == null) {
	          redirectAttributes.addFlashAttribute("message", "Please select a file to upload");
	          return "failure";
	      }

	      try {

	    	userRepository.save(user);

	      } catch (Exception e) {
	          e.printStackTrace();
	      }

	      return "success";
	  }
	  
	  
	  
	  @PostMapping("/deleteDataset") 
	  public void deleteDataset(@RequestParam("user") String userEmail,
			  @RequestParam("datasetID") int datasetID,
			  @RequestParam("datasetName") String name,
			  RedirectAttributes redirectAttributes) {
		  
		  if(userEmail != null && name != null && datasetID > 0) {
			 datasetRepository.deleteById(datasetID);
			 File file = new File(UPLOADED_FOLDER + userEmail+"/"+name);
			 if(file.exists()) {
				 FileSystemUtils.deleteRecursively(file);
			 }
		  }
	  }
	  

	  
	  @PostMapping("/getUserDatasets") // //new annotation since 4.3
	  public ArrayList<Dataset> getUserDatasets(@RequestParam("user") String userEmail,
	          RedirectAttributes redirectAttributes) {

		  ArrayList<Dataset> datasets = new ArrayList<>();
		  
		  if(userEmail != null && !userEmail.isEmpty()) {			  
			  datasets = datasetRepository.getDatasetUserEmail(userEmail);
		  }
		  

	      return datasets;
	  }
	  
	  @PostMapping("/getAllDatasets") // //new annotation since 4.3
	  public ArrayList<Dataset> getAllDatasets(
	          RedirectAttributes redirectAttributes) {

		  ArrayList<Dataset> datasets = new ArrayList<>();
		  
		  //datasets = FirebaseDataHandler.getAllDatasets();
		  
		  Iterable<Dataset> d = datasetRepository.findAll();
		  d.forEach(datasets::add);

	      return datasets;
	  }
	  
	  @PostMapping("/getDatasetsFilteredByName") // //new annotation since 4.3
	  public List<Dataset> getDatasetsFilteredByName(@RequestParam("filter") String filter,
			  @RequestParam("content") Boolean content,
	          RedirectAttributes redirectAttributes) {
		  
		 List<Dataset> datasets = new ArrayList<>();
		  if(!filter.isEmpty()) {
			  if(content) {
				  //datasets = FirebaseDataHandler.getDatasetsFilteredByName(filter.toLowerCase(),"searchContent");
				  datasets = datasetRepository.getDatasetsFiltered("dataset_search_content", "search_content", filter.toLowerCase());
			  }
			  else {
				  //datasets = FirebaseDataHandler.getDatasetsFilteredByName(filter.toLowerCase(),"searchTerms");
				  datasets = datasetRepository.getDatasetsFiltered("dataset_search_terms", "search_terms", filter.toLowerCase());
			  }
		  }
		  else {
			  //datasets = FirebaseDataHandler.getAllDatasets();
			  datasets = (List<Dataset>) datasetRepository.findAll();
		  }
		  
	      return datasets;
	  }
	  
	  @PostMapping("/getAllDataCollectionFromDB") // //new annotation since 4.3
	  public List<String> getAllDataCollectionSimple(@RequestParam("collection") String collection,
	          RedirectAttributes redirectAttributes) {

		  List<String> data = new ArrayList<>();
		  
		  if(collection.equals("categories")) {
			  ArrayList<Category> categories = new ArrayList<>();
			  Iterable<Category> d = categoryRepository.findAll();
			  d.forEach(categories::add);
			  
			  for(Category c: categories) {
				  data.add(c.getName());
			  }
			  
		  }
		  else if(collection.equals("formats")) {
			  
			  data = Stream.of(Format.values())
		              .map(Enum::name)
		              .collect(Collectors.toList());
		  }
		  //data = FirebaseDataHandler.getAllDataSimpleCollection(collection);
		  
	      return data;
	  }
	  
	  //@PostMapping("/saveDatasetMetadata") // //new annotation since 4.3
	  @RequestMapping(value = "/saveDatasetMetadata", method = RequestMethod.POST)
	  public String saveDatasetMetaData(@RequestBody Dataset dataset,
	                                 RedirectAttributes redirectAttributes) {

		  
		  
	      if (dataset == null) {
	          redirectAttributes.addFlashAttribute("message", "Please select a file to upload");
	          return "failure";
	      }

	      try {

	    	  //datasetRepository.save(d);
	    	  ArrayList<User> aux = userRepository.getUserByEmail(dataset.getOwnerEmail());
    		  
    		  if(aux.size() > 0) {
    			  dataset.setUser(aux.get(0));
    			  dataset.setOwner(aux.get(0).getName());
    			  System.out.println("Añado User al dataset --- ID --- "+ aux.get(0).getEmail());
    		  }
	    	  
	    	  
	    	  
	    	  dataset.populateTermsAndFormat();
	    	  datasetRepository.save(dataset);
	    	  
	    	  System.out.println(dataset.toString());
	    	  //dataset.populateTermsAndFormat();
	    	  //FirebaseDataHandler.saveDatasetData(dataset);
	    	  
	    	  
	    	  
//	    	  Dataset d = (Dataset) dataset;
//	    	  System.out.println(d.toString());

	      } catch (Exception e) {
	          e.printStackTrace();
	      }

	      return "success";
	  }
	  
	  @PostMapping("/test") // //new annotation since 4.3
	  public void testName(RedirectAttributes redirectAttributes) {
		  
		  
		  for(int i = 42; i < 92; i++) {
			  datasetRepository.deleteById(i);
		  }
		  
		  
		  int n = 5;
		  
		  for(int i = 0; i < n; i++) {
			  String email = "dummyuser"+i;
			  User u = new User();
			  u.setEmail(email);
			  userRepository.save(u);
			  TestData.generateRandomeData(email, 10,datasetRepository,u);
		  }
	  }
	  
	  @PostMapping("/newUser") // //new annotation since 4.3
	  public String createUser(@RequestParam("email") String email, 
			  @RequestParam("password") String password, 
			  @RequestParam("name") String name , 
			  @RequestParam("institution") String institution ,
			  @RequestParam("department") String department ,
			  @RequestParam("role") String role,
	          RedirectAttributes redirectAttributes) {

		  Dataset datasetF = new Dataset();
		  
	      if (email.isEmpty()) {
	          redirectAttributes.addFlashAttribute("message", "Please select a file to upload");
	          return "User not created";
	      }

	      try {
	    	  
	    	  if(FirebaseDataHandler.createUser(email, password)) {
	    		  
	    		  String folderPath = UPLOADED_FOLDER + email;
				  
					File directory = new File(folderPath);
				  
					if(!directory.exists())
					  directory.mkdirs();
					
					User user = new User(email, name, institution, department, role);
					userRepository.save(user);
	    		  
	    		  return "User created successfully";
	    	  }
	  
	    	  
	      } catch (Exception e) {
	          e.printStackTrace();
	      }

	      return "User not created";
	  }
	  
	  @PostMapping(path="/checkUser") // Map ONLY POST Requests
	  public @ResponseBody Boolean userInBD(@RequestParam("email") String email) {

		  Boolean userBD = false;
		  
		  ArrayList<User> aux = userRepository.getUserByEmail(email);
		  
		  if(aux.size() > 0) {
			  userBD = true;
		  }
		  
		  
	    return userBD;
	  }
	  
	  
	  @PostMapping(path="/categories") // Map ONLY POST Requests
	  public @ResponseBody String addCategories(@RequestParam String nameOld
		      , @RequestParam String email) {

		  String[] categories = {"Art","Economy","Education","Health","Politics","Science","Transport"};
		  
		  for(String cad: categories) {
			  Category c = new Category(cad);
			  
			  categoryRepository.save(c);
		  }
		  
		  
	    return "Saved";
	  }
	  
//	  @RequestMapping("/downloadDistribution")
//		public void downloadPDFResource(HttpServletRequest request, HttpServletResponse response, 
//				@RequestParam("filename") String fileName,
//				@RequestParam("dataset") String dataset,
//				@RequestParam("user") String user) throws IOException {
//
//		  	
//		  
//		  	if(dataset!=null && fileName!=null & !fileName.equals("")) {
//		  	
//		  		String filePath = UPLOADED_FOLDER+ user+"/"+ dataset+ "/" + fileName;
//		  		
//				File file = new File(filePath);
//				if (file.exists()) {
//	
//					//get the mimetype
//					String mimeType = DataGatherer.extractFileType(file);
//					if (mimeType == null) {
//						//unknown mimetype so set the mimetype to application/octet-stream
//						mimeType = "application/octet-stream";
//					}
//	
//					response.setContentType(mimeType);
//	
//					/**
//					 * In a regular HTTP response, the Content-Disposition response header is a
//					 * header indicating if the content is expected to be displayed inline in the
//					 * browser, that is, as a Web page or as part of a Web page, or as an
//					 * attachment, that is downloaded and saved locally.
//					 * 
//					 */
//	
//					/**
//					 * Here we have mentioned it to show inline
//					 */
//					//response.setHeader("Content-Disposition", String.format("inline; filename=\"" + file.getName() + "\""));
//	
//					 //Here we have mentioned it to show as attachment
//					 response.setHeader("Content-Disposition", String.format("attachment; filename=\"" + file.getName() + "\""));
//	
//					response.setContentLength((int) file.length());
//	
//					InputStream inputStream = new BufferedInputStream(new FileInputStream(file));
//	
//					FileCopyUtils.copy(inputStream, response.getOutputStream());
//	
//				}
//		  	}
//		}
	  
	  //OLD
	  @RequestMapping("/downloadDistribution/{user}/{dataset}/{fileName:.+}")
		public void downloadPDFResource(HttpServletRequest request, HttpServletResponse response,
				@PathVariable("fileName") String fileName,
				@PathVariable("user") String user,
				@PathVariable("dataset") String dataset
				) throws IOException {


		  	String filePath = UPLOADED_FOLDER+ user+"/"+ dataset+ "/" + fileName;
		  	
			File file = new File(filePath);
			if (file.exists()) {

				//get the mimetype
				String mimeType = DataGatherer.extractFileType(file);
				if (mimeType == null) {
					//unknown mimetype so set the mimetype to application/octet-stream
					mimeType = "application/octet-stream";
				}

				response.setContentType(mimeType);

				/**
				 * In a regular HTTP response, the Content-Disposition response header is a
				 * header indicating if the content is expected to be displayed inline in the
				 * browser, that is, as a Web page or as part of a Web page, or as an
				 * attachment, that is downloaded and saved locally.
				 * 
				 */

				/**
				 * Here we have mentioned it to show inline
				 */
				//response.setHeader("Content-Disposition", String.format("inline; filename=\"" + file.getName() + "\""));

				 //Here we have mentioned it to show as attachment
				 response.setHeader("Content-Disposition", String.format("attachment; filename=\"" + file.getName() + "\""));

				response.setContentLength((int) file.length());

				InputStream inputStream = new BufferedInputStream(new FileInputStream(file));

				FileCopyUtils.copy(inputStream, response.getOutputStream());

			}
		}


}
