package datahub.model;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.Lob;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonManagedReference;

import datahub.tools.DataGatherer;

@Entity(name = "Dataset")
@Table(name = "dataset")
public class Dataset {

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int id;
	
	// Variables
	@Column(name = "name")
	private String name;
	
	@Lob
	@Column(name = "description")
	private String description;
	
	@ElementCollection(targetClass=String.class)
	private List<String> keywords;
	@ElementCollection(targetClass=String.class)
	private List<String> formats;
	@ElementCollection(targetClass=String.class)
	private List<String> searchTerms;
	@ElementCollection(targetClass=String.class)
	private List<String> searchContent;
	@Column(name = "category")
	private String category;
	@Column(name = "date_created")
	private String dateCreated;
	@Column(name = "date_updated")
	private String dateUpdated;
	@Column(name = "license")
	private String license;
	@Column(name = "version")
	private String version;
	@Column(name = "owner")
	private String owner;
	@Column(name = "owner_email")
	private String ownerEmail;
	
	@OneToMany(
        mappedBy = "dataset",
        cascade = CascadeType.ALL,
        orphanRemoval = true,
        fetch = FetchType.LAZY
    )
	@JsonManagedReference
	private Set<DatasetFile> distributions;
	
	@ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "user_id", referencedColumnName = "id")
	@JsonBackReference
    private User user;
	
	// Constrcutors
	public Dataset() {
		super();
		this.name = "";
		this.description = "";
		this.keywords = new ArrayList<String>();
		this.formats = new ArrayList<String>();
		this.searchTerms = new ArrayList<String>();
		this.searchContent = new ArrayList<String>();
		this.category = "";
		this.dateCreated = "";
		this.dateUpdated = "";
		this.license = "";
		this.version = "";
		this.owner = "";
		this.ownerEmail = "";
		this.distributions = new HashSet<>();
	}
	
	public Dataset(String name, String description, ArrayList<String> keywords, String category, String dateCreated,
			String dateUpdated, String license, String version, String owner, String ownerEmail, Set<DatasetFile> distributions) {
		super();
		this.name = name;
		this.description = description;
		this.keywords = keywords;
		this.category = category;
		this.dateCreated = dateCreated;
		this.dateUpdated = dateUpdated;
		this.license = license;
		this.version = version;
		this.owner = owner;
		this.ownerEmail = ownerEmail;
		this.distributions = distributions;
	}

	// Setters and Getters
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public List<String> getKeywords() {
		return keywords;
	}

	public void setKeywords(ArrayList<String> keywords) {
		this.keywords = keywords;
	}

	public String getCategory() {
		return category;
	}

	public void setCategory(String category) {
		this.category = category;
	}

	public String getDateCreated() {
		return dateCreated;
	}

	public void setDateCreated(String dateCreated) {
		this.dateCreated = dateCreated;
	}

	public String getDateUpdated() {
		return dateUpdated;
	}

	public void setDateUpdated(String dateUpdated) {
		this.dateUpdated = dateUpdated;
	}

	public String getLicense() {
		return license;
	}

	public void setLicense(String license) {
		this.license = license;
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public String getOwner() {
		return owner;
	}

	public void setOwner(String owner) {
		this.owner = owner;
	}

	public Set<DatasetFile> getDistributions() {
		return distributions;
	}

	public void setDistributions(Set<DatasetFile> distributions) {
		this.distributions = distributions;
	}
	
	public String getOwnerEmail() {
		return ownerEmail;
	}

	public void setOwnerEmail(String ownerEmail) {
		this.ownerEmail = ownerEmail;
	}
	
	public List<String> getFormats() {
		return formats;
	}

	public void setFormats(ArrayList<String> formats) {
		this.formats = formats;
	}

	public List<String> getSearchTerms() {
		return searchTerms;
	}

	public void setSearchTerms(ArrayList<String> searchTerms) {
		this.searchTerms = searchTerms;
	}
	
	public List<String>  getSearchContent() {
		return searchContent;
	}

	public void setSearchContent(ArrayList<String>  searchContent) {
		this.searchContent = searchContent;
	}
	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	// Methods
	public void addDistributions(ArrayList<DatasetFile> datasets) {
		this.distributions.addAll(datasets);
	}
	
	public void clearTerms() {
		searchTerms.clear();
		searchContent.clear();
	}
	
	public void populateTermsAndFormat() {
		formats.clear();
		searchTerms.clear();
		searchContent.clear();
		
		Set<String> content = new HashSet<>();
		Set<String> f = new HashSet<>();
		
		for(DatasetFile d: distributions) {
			f.add(d.getFormat().toString());
			content.addAll(DataGatherer.getDistributionContent(d, ownerEmail, name));
		}
		
		formats.addAll(f);
		
		for(String s: keywords) {
			searchTerms.add(s.toLowerCase());		
		}
		
		String[] auxTerms = name.split("\\s+");
		
		for(String s: auxTerms) {
			searchTerms.add(s.toLowerCase());		
		}
		
		
		content.addAll(searchTerms);
		
		searchContent.addAll(content);
		
	}
	
	@Override
	public String toString() {
		return "Dataset [name=" + name + ", description=" + description + ", keywords=" + keywords + ", category="
				+ category + ", dateCreated=" + dateCreated + ", dateUpdated=" + dateUpdated + ", license=" + license
				+ ", version=" + version + ", owner=" + owner + ", distributions=" + distributions + "]";
	}
	

	
	
	
	
	
}
