package datahub.model;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import com.fasterxml.jackson.annotation.JsonBackReference;
import com.fasterxml.jackson.annotation.JsonManagedReference;

@Entity(name = "DatasetFile")
@Table(name = "dataset_file")
public class DatasetFile {

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private int id;
	
	// Variables
	@Column(name = "file_name")
	private String fileName;
	@Column(name = "title")
	private String title;
	@Column(name = "format")
	private Format format;
	@Column(name = "size")
	private long size; // in bytes
	
	@OneToMany(
        mappedBy = "datasetFile",
        cascade = CascadeType.ALL,
        orphanRemoval = true,
        fetch = FetchType.LAZY
    )
	@JsonManagedReference
	private Set<Label> labels;
	
	@ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "dataset_id", referencedColumnName = "id")
	@JsonBackReference
    private Dataset dataset;
	
	
	// Constructor
	public DatasetFile() {
		super();
		this.fileName = "";
		this.format = Format.CSV;
		this.size = 0;
		this.labels = new HashSet<>();
		this.title = "";
	}
	
	public DatasetFile(String fileName, Format format, long size, Set<Label> labels, String title) {
		super();
		this.fileName = fileName;
		this.format = format;
		this.size = size;
		this.labels = labels;
		this.title = title;
	}

	// Getters and Setters
	public String getFileName() {
		return fileName;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public Format getFormat() {
		return format;
	}

	public void setFormat(Format format) {
		this.format = format;
	}

	public long getSize() {
		return size;
	}

	public void setSize(long size) {
		this.size = size;
	}

	public Set<Label> getLabels() {
		return labels;
	}
	
	public void setLabels(Set<Label> labels) {
		this.labels = labels;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}
	
	public Dataset getDataset() {
		return dataset;
	}

	public void setDataset(Dataset dataset) {
		this.dataset = dataset;
	}

	
	// Methods
	


	// Override Methods
	@Override
	public String toString() {
		return "DatasetFile [fileName=" + fileName + ", format=" + format + ", size=" + size + ", labels=" + labels.toString()
				+ "]\n";
	}
	
}
